//
// Circles Demo
//
// GameX (c) R.C. Hoetzlein 2003
// 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <process.h>

#define GAMEX_MAIN
#include "gamex.hpp"

#define LAST_FRAME			600				// Last frame to be measured

#define MAX_CIRCLES			1000			// Maximum number of circles allowed
#define CIRCLE_RADIUS		20				// Circle radius in pixels

#define MIN_X				0				// Minimum X-coordinate of circles
#define MIN_Y				0				// Minimum Y-coordinate of circles
#define MAX_X				800				// Maximum X-coordinate of circles
#define MAX_Y				500				// Maximum Y-coordinate of circles

struct Circle {								//** Single circle
	float x, y;								// Position of a circle	
	float dx, dy;							// Velocity of a circle
};

class CircleSet {							//** Set of circles
public:
	CircleSet ();
	void Initialize (int n);
	void AddCircle (float newx, float newy, float newdx, float newdy);
	void MoveCircles (void);
	void DrawCircles (void);

private:
	ImageX	circle_img;
	int		num_list;			
	Circle	list[MAX_CIRCLES];
};

CircleSet::CircleSet ()						// Circle set constructor
{
	num_list = 0;							// Start with 0 circles
}

void CircleSet::Initialize (int num_start)
{
	float nx, ny, dx, dy;
	
	ImageExt imgext;						// Load circle image
	imgext.Load ("circle.tif", &circle_img, DRAW_MASKED);	

	srand (time (NULL));	
	for (int n=0; n < num_start; n++) {		// Create 100 circles to start with,
		nx = rand()*MAX_X / RAND_MAX;		// placing them at random locations
		ny = rand()*MAX_Y / RAND_MAX;		// and giving them random velocities.
		dx = (rand()*2.0f / RAND_MAX)-1.0f;
		dy = (rand()*2.0f / RAND_MAX)-1.0f;
		AddCircle (nx, ny, dx, dy);
	}
}
											// Add a single circle to the set
void CircleSet::AddCircle (float newx, float newy, float newdx, float newdy)
{
	if (num_list < MAX_CIRCLES) {			// If there is room to add another...
		list[num_list].x = newx;			
		list[num_list].y = newy;			// Add the new circle position and velocity
		list[num_list].dx = newdx;			// to the list of circles.
		list[num_list].dy = newdy;
		num_list++;							// Increment number of circles
	}
}

void CircleSet::MoveCircles (void)			// Update the positions of all circles
{
	for (int n=0; n < num_list; n++) {			
		list[n].x += list[n].dx;
		list[n].y += list[n].dy;		
	}
}

void CircleSet::DrawCircles (void)			// Draw all circles in the set
{
	for (int n=0; n < num_list; n++)
		GameX.DrawMasked (list[n].x, list[n].y, &circle_img);
}

//*********** MAIN PROGRAM


int frame;									// Current frame
int start, stop;							// Elapsed time
CircleSet circles;							// Circle Set

// GameX initialize function (called when program starts)
void initialize (void)
{
	GameX.Initialize (false, 800, 500);		// Initialize GameX itself (required)
	circles.Initialize (100);
	frame=0;
	start = clock();
}

// GameX main function (called repeatedly as fast as possible)
void main (void)
{
	GameX.DrawOpen ();						// Draw circles (always)
		GameX.DrawClear ();
		circles.DrawCircles ();
	GameX.DrawClose ();

	if (frame < LAST_FRAME) {				// If frame is less than LAST_FRAME..
		char disp[50];
		sprintf (disp, "Frame: %d", frame);	// Draw frame counter.
		GameX.DrawColor (255, 255, 255);
		GameX.DrawText (10, 10, disp);

		circles.MoveCircles();				// Move the circles.
		frame++;							// Count the frames.
	} else {
		if (frame==LAST_FRAME) {stop = clock(); frame++;}
		char disp[50];						// If frame is 600...
		sprintf (disp, "Frame: %d", frame);	// Draw frame counter.
		GameX.DrawColor (255, 255, 255);
		GameX.DrawText (10, 10, disp);		
		sprintf (disp, "Elapsed Time: %4.3f secs", (float) (stop-start)/CLOCKS_PER_SEC);	// Draw elapsed time
		GameX.DrawColor (255, 255, 255);
		GameX.DrawText (10, 40, disp);
	}	
	
	GameX.ViewUpdate ();					// Update view with newly drawn image (required)
}


